@@ -20,13 +20,18 @@ from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes, mark_inset
 from beamprofiler.utils import data_processing as dp
 
 
-def general_plot():
+def general_plot(proj=None):
     """
     `general_plot` returns a general-purpose, blank pyplot graph.
 
+    Parameters
+    ----------
+    proj : str, optional
+        Axes projection. The default is None.
+
     Returns
     -------
-    fig : Figure object of matplotlib.figure module
+    fig : figure object of matplotlib.figure module
         blank matplotlib figure object.
     ax : AxesSubplot object of matplotlib.axes_subplots module
         blank matplotlib axes object.
@@ -40,7 +45,7 @@ def general_plot():
     plt.rcParams.update({'font.size': 6})
 
     fig = plt.figure(figsize=(5, 5 / golden_ratio), dpi=100)
-    ax = fig.add_subplot(1, 1, 1)
+    ax = fig.add_subplot(1, 1, 1, projection=proj)
 
     return fig, ax
 
@@ -54,23 +59,25 @@ def histogram(path, fileName, beam, **kwargs):
     ----------
     path : std
         path to where the graph will be saved.
+    fileName : str
+        name of the power density distribution file.
     beam : Beam
         instance of type Beam.
 
     Other Parameters
     ----------------
     n_bins : int
-        number of bins used in the histogram.
+        number of bins used in the histogram. The default is 256.
     zoom : float
-        zoom of the inset image.
+        zoom of the inset image. The default is 2.
     x1 : float
-        lower bound of the inset image on the x-axis.
+        lower bound of the inset image on the x-axis. The default is 1600.
     x2 : float
-        upper bound of the inset image on the y-axis.
+        upper bound of the inset image on the y-axis. The default is 2000.
     y1 : float
-        lower bound of the inset image on the y-axis.
+        lower bound of the inset image on the y-axis. The default is 0.
     y2 : float
-        upper bound of the inset image on the x-axis.
+        upper bound of the inset image on the x-axis. The default is 5000.
 
     Returns
     -------
@@ -79,7 +86,7 @@ def histogram(path, fileName, beam, **kwargs):
     """
 
     # Check if any default value has been redefined in kwargs
-    n_bins = kwargs.pop('bins', 256)
+    n_bins = kwargs.pop('n_bins', 256)
     zoom = kwargs.pop('zoom', 2)
     x1 = kwargs.pop('x1', 1600)
     x2 = kwargs.pop('x2', 2000)
@@ -108,7 +115,7 @@ def histogram(path, fileName, beam, **kwargs):
     inset.tick_params(labelleft=False, labelbottom=False)
     mark_inset(ax, inset, loc1=3, loc2=3, fc="none", ec="0.5")
 
-    # Get the highest histogram peak around the high-poewr density area (upper
+    # Get the highest histogram peak around the high-power density area (upper
     # 40% of the number of bins)
     low_bound = int(n_bins * 0.6)
     high_bound = n_bins
@@ -127,28 +134,32 @@ def histogram(path, fileName, beam, **kwargs):
     plt.show()
 
 
-def heat_map(path, fileName, beam, **kwargs):
+def heat_map_2d(path, fileName, beam, **kwargs):
     """
-    `heat_map` plots the heat map of the power densidty distribution.
+    `heat_map_2d` plots the 2D heat map of the power density distribution.
 
     Parameters
     ----------
     path : str
-        path to where the graph will be saved..
+        path to where the graph will be saved.
+    fileName : str
+        name of the power density distribution file.
     beam : Beam
         instance of type `Beam`.
 
     Other Parameters
     ----------------
     z_lim : float
-        upper intensity limite of the cross-section graph of the power density
-        distribution.
+        upper intensity limit of the cross-section graph of the power density
+        distribution. The default is -1.
     cross_x : float
         x-coordinate of the cross-section graph of the power density
-        distribution
+        distribution. The default is the calculated beam center about the
+        x-axis.
     cross_y : float
         y-coordinate of the cross-section graph of the power density
-        distribution
+        distribution. The default is the calculated beam center about the
+        y-axis.
 
     Returns
     -------
@@ -224,12 +235,88 @@ def heat_map(path, fileName, beam, **kwargs):
 
     # Save and show
     fileName = os.path.splitext(fileName)[0]
-    plt.savefig(os.path.join(path, fileName + ' - heat map.png'),
+    plt.savefig(os.path.join(path, fileName + ' - 2d heat map.png'),
+                bbox_inches='tight', dpi=300)
+    plt.show()
+
+
+def heat_map_3d(path, fileName, beam, **kwargs):
+    """
+    `heat_map_3d` plots the 3D heat map of the power density distribution.
+
+    Parameters
+    ----------
+    path : str
+        path to where the graph will be saved.
+    fileName : str
+        name of the power density distribution file.
+    beam : Beam
+        instance of type `Beam`.
+
+    Other Parameters
+    ----------------
+    elev : float
+        elevation viewing angle. The default is 50.
+    azim : float
+        azimuthal viewing angle. The default is 135.
+    dist : float
+        distance from the plot. The default is 11.
+
+    Returns
+    -------
+    None.
+
+    """
+
+    # Check if any default value has been redefined in kwargs
+    elev = kwargs.pop('elev', 50)
+    azim = kwargs.pop('azim', 135)
+    dist = kwargs.pop('dist', 11)
+
+    fig, ax = general_plot(proj='3d')
+
+    # Configure view
+    ax.view_init(elev=elev, azim=azim)
+    ax.dist = dist
+
+    # Plot data
+    x = np.mgrid[0:dp.get_xWindow(beam.raw_header):beam.xResolution]
+    y = np.mgrid[0:dp.get_yWindow(beam.raw_header):beam.yResolution]
+    x_3d, y_3d = np.meshgrid(x, y)
+    ax.plot_surface(x_3d, y_3d, beam.raw_data, cmap=cm.gist_rainbow_r,
+                    rstride=2, cstride=2, linewidth=2, antialiased=False)
+
+    # Set axis labels
+    ax.set_xlabel("x-axis (mm)", labelpad=5)
+    ax.set_ylabel("y-axis (mm)", labelpad=5)
+    ax.set_zlabel("Intensity", labelpad=5)
+
+    # Save and show
+    fileName = os.path.splitext(fileName)[0]
+    plt.savefig(os.path.join(path, fileName + ' - 3d heat map.png'),
                 bbox_inches='tight', dpi=300)
     plt.show()
 
 
 def norm_energy_curve(path, fileName, beam):
+    """
+    `norm_energy_curve` plots the normalized energy curve of the power density
+    distribution.
+
+    Parameters
+    ----------
+    path : str
+        path to where the graph will be saved.
+    fileName : str
+        name of the power density distribution file.
+    beam : Beam
+        instance of type `Beam`.
+
+    Returns
+    -------
+    None.
+
+    """
 
     # Get the figure and axes objects
     fig, ax = general_plot()
@@ -282,7 +369,7 @@ def norm_energy_curve(path, fileName, beam):
 
     # Save and show
     fileName = os.path.splitext(fileName)[0]
-    plt.savefig(os.path.join(path, fileName + ' - top-hat factor.png'),
+    plt.savefig(os.path.join(path, fileName + ' - energy curve.png'),
                 bbox_inches='tight', dpi=300)
     plt.show()
 
@@ -308,5 +395,6 @@ def run(path, fileName, beam, **kwargs):
     """
 
     histogram(path, fileName, beam, **kwargs)
-    heat_map(path, fileName, beam, **kwargs)
+    heat_map_2d(path, fileName, beam, **kwargs)
+    heat_map_3d(path, fileName, beam, **kwargs)
     norm_energy_curve(path, fileName, beam)
